--[[
A SecureHeader presents an interface to the the Blizzard/WoW secure header
frame for the functionality that UAB needs.
--]]
uab.classes.SecureHeader = uab.mixins.aceOO.Class()

function uab.classes.SecureHeader.prototype:init()
	uab.classes.SecureHeader.super.prototype.init(self)

    self.stateparent = {}
    self.stateparentrange = {}
    self.statescale = {}
    self.statescalerange = {}
    self.statepos = {}
    self.stateposrange = {}

	--[[
	Create the header frame.
	--]]
    
    self.frame = CreateFrame("Frame", "uabHeader", UIParent, "SecureStateHeaderTemplate")
    self.frame.uabobj = self
    self.frame:SetAttribute("state", "0")
    self.frame.uabobj = self

    self.frame:SetAttribute("useparent-unit", "true")
    self.frame:SetAttribute("useparent-unitsuffix", "true")

    self:SetStateParent()
    self:SetStatePosition()
    self:SetStateScale()
end

function uab.classes.SecureHeader.prototype:IAmDead()
    --[[
    Set the header frame state attribute to 0 to (hopefully) cause our cancel and action buttons
    to hide (because of the state change to a hidden state).  Since we are dead, we are no longer
    in combat, so this *should* be ok to do.
    --]]
    self.frame:SetAttribute("state", "0")
end

function uab.classes.SecureHeader.prototype:SetStateParent(state, frame)
    if state then
        if frame then
            uab.pdebug("SetStateParent        (set)  : ("..state..") : "..frame:GetName())
        else
            uab.pdebug("SetStateParent        (clear): ("..state..")")
        end
        
        if frame then
            self.stateparent[state] = frame
        else
            self.frame:SetAttribute("headparent-hps"..state, nil)
            self.stateparent[state] = nil
        end
    end
    
    self:SetStateParentAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStateParentRange(minstate, maxstate, frame)
    if minstate and maxstate then
        if frame then
            uab.pdebug("SetStateParentRange   (set)  : ("..state..") : "..frame:GetName())
        else
            uab.pdebug("SetStateParentRange   (clear): ("..state..")")
        end
        
        local idxtodel
        for idx, range in pairs(self.stateparentrange) do
            if range.minstate == minstate and range.maxstate == maxstate then
                idxtodel = idx
                break
            end
        end

        if idxtodel then
            self.stateparentrange[idxtodel] = nil
        end

        if frame then
            table.insert(self.stateparentrange, {minstate = minstate, maxstate = maxstate, frame = frame})
        end
    end
    
    self:SetStateParentAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStateScale(state, scale)
    if state then
        if scale then
            uab.pdebug("SetStateScale         (set)  : ("..state..") : "..scale)
        else
            uab.pdebug("SetStateScale         (clear): ("..state..")")
        end
        
        if scale then
            self.statescale[state] = scale
        else
            self.statescale[state] = nil
        end
    end
    
    self:SetStateScaleAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStateScaleRange(minstate, maxstate, scale)
    if minstate and maxstate then
        if scale then
            uab.pdebug("SetStateScaleRange    (set)  : ("..minstate..","..maxstate..") : "..scale)
        else
            uab.pdebug("SetStateScaleRange    (clear): ("..minstate..","..maxstate..")")
        end
        
        local idxtodel
        for idx, range in pairs(self.statescalerange) do
            if range.minstate == minstate and range.maxstate == maxstate then
                idxtodel = idx
                break
            end
        end

        if idxtodel then
            --uab.print("Found scale range... removing at index: "..idxtodel)
            self.statescalerange[idxtodel] = nil
        end

        if scale then
            table.insert(self.statescalerange, { minstate = minstate, maxstate = maxstate, scale = scale })
        end
    end
    
    self:SetStateScaleAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStatePosition(state, x, y, relpoint, point)
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    if state then
        if x and y and relpoint and point then
            uab.pdebug("SetStatePosition      (set)  : ("..state..") : ("..x..","..y..","..relpoint..","..point..")")
        else
            uab.pdebug("SetStatePosition      (clear): ("..state..")")
        end
        
        if x and y and relpoint and point then
            self.statepos[state] = {}
            self.statepos[state].point = point
            self.statepos[state].relpoint = relpoint
            self.statepos[state].x = x
            self.statepos[state].y = y
        else
            self.statepos[state] = nil
        end
    end

    self:SetStatePositionAttributes()
end

function uab.classes.SecureHeader.prototype:SetStatePositionRange(minstate, maxstate, x, y, relpoint, point)
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    if minstate and maxstate then
        if x and y and relpoint and point then
            uab.pdebug("SetStatePositionRange (set)  : ("..minstate..","..maxstate..") : ("..x..","..y..","..relpoint..","..point..")")
        else
            uab.pdebug("SetStatePositionRange (clear): ("..minstate..","..maxstate..")")
        end
        
        local idxtodel
        for idx, range in pairs(self.stateposrange) do
            if range.minstate == minstate and range.maxstate == maxstate then
                idxtodel = idx
                break
            end
        end

        if idxtodel then
            self.stateposrange[idxtodel] = nil
        end

        if x and y and relpoint and point then
            table.insert(self.stateposrange, { minstate = minstate, maxstate = maxstate, point = point, relpoint = relpoint, x = x, y = y })
        end
    end

    self:SetStatePositionAttributes()
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.SecureHeader.prototype:SetStateParentAttributes()
    local headparents
    local parentcount = 1

    for state, parentframe in pairs(self.stateparent) do
        if headparents then
            headparents = headparents .. ";"
        else
            headparents = ""
        end

        self.frame:SetAttribute("headparent-hps"..parentcount, parentframe)
        
        headparents = headparents .. state .. ":hps" .. parentcount
        parentcount = parentcount + 1
    end

    for _, range in pairs(self.stateparentrange) do
        if headparents then
            headparents = headparents .. ";"
        else
            headparents = ""
        end

        self.frame:SetAttribute("headparent-hps"..parentcount, range.frame)
        
        headparents = headparents .. range.minstate .. "-" .. range.maxstate .. ":hps" .. parentcount
    end

    self.frame:SetAttribute("headparent", headparents)
end

function uab.classes.SecureHeader.prototype:SetStateScaleAttributes()
    local headscale

    for state, statescale in pairs(self.statescale) do
        if not headscale then
            headscale = ""
        else
            headscale = headscale .. ";"
        end

        headscale = headscale .. state .. ":" .. statescale
    end
    
    for _, range in pairs(self.statescalerange) do
        if not headscale then
            headscale = ""
        else
            headscale = headscale .. ";"
        end

        headscale = headscale .. range.minstate .. "-" .. range.maxstate .. ":" .. range.scale
    end

    if headscale then
        headscale = headscale .. ";*:" .. uab.db.profile.scale.default.value
    else
        headscale = "*:" .. uab.db.profile.scale.default.value
    end

    uab.pdebug("headscale:       "..headscale)
    self.frame:SetAttribute("headscale", headscale)
end

function uab.classes.SecureHeader.prototype:SetStatePositionAttributes()
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    for state, pos in pairs(self.statepos) do
        if not headofspoint then
            headofspoint = ""
            headofsrelpoint = ""
            headofsx = ""
            headofsy = ""
        else
            headofspoint = headofspoint .. ";"
            headofsrelpoint = headofsrelpoint .. ";"
            headofsx = headofsx .. ";"
            headofsy = headofsy .. ";"
        end

        headofspoint = headofspoint .. state .. ":" .. pos.point
        headofsrelpoint = headofsrelpoint .. state .. ":" .. pos.relpoint
        headofsx = headofsx .. state .. ":" .. pos.x
        headofsy = headofsy .. state .. ":" .. pos.y
    end

    for _, range in pairs(self.stateposrange) do
        if not headofspoint then
            headofspoint = ""
            headofsrelpoint = ""
            headofsx = ""
            headofsy = ""
        else
            headofspoint = headofspoint .. ";"
            headofsrelpoint = headofsrelpoint .. ";"
            headofsx = headofsx .. ";"
            headofsy = headofsy .. ";"
        end

        local staterange = range.minstate .. "-" .. range.maxstate
        
        headofspoint = headofspoint .. staterange .. ":" .. range.point
        headofsrelpoint = headofsrelpoint .. staterange .. ":" .. range.relpoint
        headofsx = headofsx .. staterange .. ":" .. range.x
        headofsy = headofsy .. staterange .. ":" .. range.y
    end

    if headofspoint then
        headofspoint = headofspoint .. ";*:CENTER"
        headofsrelpoint = headofsrelpoint .. ";*:cursor"
        headofsx = headofsx .. ";*:0"
        headofsy = headofsy .. ";*:0"
    else
        headofspoint = "*:" .. uab.db.profile.position.default.point
        headofsrelpoint = "*:" .. uab.db.profile.position.default.relpoint
        headofsx = "*:" .. uab.db.profile.position.default.x
        headofsy = "*:" .. uab.db.profile.position.default.y
    end

    uab.pdebug("headofspoint:    "..headofspoint)
    uab.pdebug("headofsrelpoint: "..headofsrelpoint)
    uab.pdebug("headofsx:        "..headofsx)
    uab.pdebug("headofsy:        "..headofsy)
--[[
--]]
    
    self.frame:SetAttribute("headofspoint", headofspoint)
    self.frame:SetAttribute("headofsrelpoint", headofsrelpoint)
    self.frame:SetAttribute("headofsx", headofsx)
    self.frame:SetAttribute("headofsy", headofsy)
end

function uab.classes.SecureHeader.prototype:SetScale(frame)
    if frame then
        self.scaleframe = frame
    end

    local relativeScale = self:RelativeScale()

    self.frame:SetScale(relativeScale)
end

function uab.classes.SecureHeader.prototype:RelativeScale()
    local relativeScale = self.group:GetScale()

    if self.scaleframe then
        local frame = self.scaleframe

        while frame:GetParent() do
            local nextref = frame:GetParent()
            if nextref then
                if nextref ~= UIParent then
                    frame = nextref
                else
                    do break end
                end
            end
        end
    
        local scale = 1.0 / frame:GetScale()
        
        relativeScale = scale * relativeScale
    end
    
    return relativeScale
end

