--[[
A UnitGroupHeader maintains a set of ActionButton and ActionButtonNone
instances.  It controls the state transitions for those buttons.  A
UnitGroupHeader also maintains a set of UnitGroupControl instances.  In WoW
2.01, there is a single UnitGroupControl for each UnitGroupHeader.  In WoW
2.02, there will be a UnitGroupControl for each unit in the associated
UnitGroup.

The UnitGroupHeader defines the valid states, and button-unit associations for
each of the ActionButton instances it maintains.  Each ActionButton is told
the state-unit associations, so that when the button is clicked, the action
associated with the button applies to the correct unit based on the current
state and those associations.

The UnitGroupHeader is associated with a single UnitGroup
The UnitGroupHeader maintains a list of ActionButton instances
The UnitGroupHeader maintains a single ActionButtonNone instance
The UnitGroupHeader maintains a single ActionSetLayout instance
--]]
uab.classes.UnitGroupHeader = uab.mixins.aceOO.Class()
uab.classes.UnitGroupHeader.nextState = 1

function uab.classes.UnitGroupHeader.NextState()
    local nextState = uab.classes.UnitGroupHeader.nextState
    uab.classes.UnitGroupHeader.nextState = uab.classes.UnitGroupHeader.nextState + 1

    return nextState
end

function uab.classes.UnitGroupHeader.prototype:init(group, numunits, actions)
	uab.classes.UnitGroupHeader.super.prototype.init(self)

	self.group = group
    self.numcontrols = numunits * uab.constants.controlsperunit
	self.controls = {}
    
	--[[
	Create the header frame.  We create one header from (the only one for the group), and
	multiple controls, and the loop counter is passed to the control to differentiate
    between control instances.
	--]]

    self.frame = uab.main.secureheader.frame
--[[
    self.stateparents = {}

    self.frame = CreateFrame("Frame", "uab" .. self.group.activator.name .. self.group.name .. "Hdr", UIParent, "SecureStateHeaderTemplate")
    self.frame.uabobj = self
    self.frame:SetAttribute("state", "0")
    self.frame.uabheader = self

    self.frame:SetAttribute("headofsrelpoint", "cursor")
    self.frame:SetAttribute("useparent-unit", "true")
    self.frame:SetAttribute("useparent-unitsuffix", "true")
--]]
    
    self.states = {}
    
    local state
    
    for idx = 1, self.numcontrols, 1 do
        state = uab.classes.UnitGroupHeader.NextState()
        if not self.states.min then
            self.states.min = state
        end
        self.controls[idx] = uab.classes.UnitGroupControl:new(self, state, group.name)
    end

    self.states.max = state

    uab.pdebug("Header: "..self.group.activator.name .. self.group.name.." : staterange = "..self.states.min.."-"..self.states.max)
    
	self.cancelbtn = uab.classes.ActionButtonNone:new(self, self.states.min,self.states.max)
	self.actionbtns = {}

    if actions then
    	for idx = 1, 24, 1 do
    	    if actions[idx] then
    	        self:SetAction(idx, actions[idx])
    		end
    	end
    end

    self:PositionChanged()
    self:ScaleChanged()
end

function uab.classes.UnitGroupHeader.prototype:IAmDead()
    --[[
    Set the header frame state attribute to 0 to (hopefully) cause our cancel and action buttons
    to hide (because of the state change to a hidden state).  Since we are dead, we are no longer
    in combat, so this *should* be ok to do.
    --]]
    uab.main.secureheader:IAmDead()
    self.cancelbtn:Hide()
    for _, btn in pairs(self.actionbtns) do
        btn:Hide()
    end
end

function uab.classes.UnitGroupHeader.prototype:SetAction(number, action)
    if not self.actionbtns[number] then
	    self.actionbtns[number] = uab.classes.ActionButton:new(self, self.states.min, self.states.max, action)
	    for _, control in pairs(self.controls) do
			self.actionbtns[number]:AddActiveStateUnit(control.state, control.unit)
		end
    else
	    self.actionbtns[number]:SetAction(action)
        self.actionbtns[number]:Enable()
    end
end

function uab.classes.UnitGroupHeader.prototype:LayoutChanged()
    if self.group.layout then
        self.group.layout:ArrangeButtons(self.cancelbtn, self.actionbtns)
    end
end

function uab.classes.UnitGroupHeader.prototype:IsRegisteredFrame(frame)
    for _, control in pairs(self.controls) do
		if control:IsRegisteredFrame(frame) then
            return true
        end
	end
end

function uab.classes.UnitGroupHeader.prototype:UnregisterFrame(frame)
    for _, control in pairs(self.controls) do
		if control:IsRegisteredFrame(frame) then
            control:UnassignFrame()
            return true
        end
	end
end

function uab.classes.UnitGroupHeader.prototype:CanRegisterFrame(frame)
    local unit = SecureButton_GetModifiedUnit(frame)
    local canregister
    local groupname = self.group.name
    
	if not unit and groupname == "partyraid" then
		canregister = true
	elseif unit then
        canregister = (groupname == unit)
        if not canregister then
            if string.sub(unit, 1, 5) == "party" then
            	canregister = (groupname == "partyraid")
            elseif string.sub(unit, 1, 4) == "raid" then
            	canregister = (groupname == "partyraid")
            end
        end
	end

	return canregister and self:HaveAvailableControl()
end

function uab.classes.UnitGroupHeader.prototype:RegisterFrame(frame)
    local control = self:NextControl()

    if not control then return end

    --[[
    Inform this control of all the valid states so that when
    the control for another state is selected, the buttons will
    reassosicate with the other state instead of this one, and
    thereby associate with the other unit frame.
    --]]

    for _, ctrl in pairs(self.controls) do
        if ctrl.unitframe then
            ctrl:StatesInUseChanged()
        end
    end
            
    self.cancelbtn:AddActiveStateUnit(control.state, control.unit)
	
    control:AssignFrame(frame)
end

function uab.classes.UnitGroupHeader.prototype:ResetButtons()
	self.cancelbtn:Hide()

	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:Hide()
	end	
    
    self:LayoutChanged()
end

function uab.classes.UnitGroupHeader.prototype:Enable(force)
    if force or not self.forcedisable then
        self.forcedisable = nil
        self.cancelbtn:Enable()
        for _, control in pairs(self.controls) do
    		control:Enable()
    	end
    end
end

function uab.classes.UnitGroupHeader.prototype:Disable(force)
    self.forcedisable = force or self.forcedisable
    self.cancelbtn:Disable()
    for _, control in pairs(self.controls) do
		control:Disable()
	end
end

function uab.classes.UnitGroupHeader.prototype:SetFrameGroupFrameEnabledStates()
	for _, control in pairs(self.controls) do
	    control:SetEnabledState()
	end
end

function uab.classes.UnitGroupHeader.prototype:SwitchToSingleAction(action)
    for _, control in pairs(self.controls) do
        control:SwitchToDirectAction(action)
    end
end

function uab.classes.UnitGroupHeader.prototype:SwitchToButtons()
    for _, control in pairs(self.controls) do
        control:SwitchToClickAction()
    end
end

function uab.classes.UnitGroupHeader.prototype:ShowActions()
	self.cancelbtn:Show()

	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:Show()
	end	
end

function uab.classes.UnitGroupHeader.prototype:TimerEvent(elapsed)
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:TimerEvent(elapsed)
	end	
end

function uab.classes.UnitGroupHeader.prototype:SetStateParent(state, frame)
    uab.main.secureheader:SetStateParent(state, frame)
--[[
    local headparents

    self.stateparents[state] = frame
    
    for state, parentframe in pairs(self.stateparents) do
        if headparents then
            headparents = headparents .. ";"
        else
            headparents = ""
        end

        self.frame:SetAttribute("headparent-hps"..state, parentframe)
        
        headparents = headparents .. state .. ":hps" .. state
    end

    self.frame:SetAttribute("headparent", headparents)
--]]
end

function uab.classes.UnitGroupHeader.prototype:ClearStateParent(state)
    uab.main.secureheader:SetStateParent(state)
--[[
    local headparents

    self.frame:SetAttribute("headparent-hps"..state, nil)
    
    self.stateparents[state] = nil
    
    for state, parentframe in pairs(self.stateparents) do
        if headparents then
            headparents = headparents .. ";"
        else
            headparents = ""
        end

        self.frame:SetAttribute("headparent-hps"..state, parentframe)
        
        headparents = headparents .. state .. ":hps" .. state
    end

    self.frame:SetAttribute("headparent", headparents)
--]]
end

function uab.classes.UnitGroupHeader.prototype:ScaleChanged()
    for _, control in pairs(self.controls) do
        control:ScaleChanged()
    end

    if self.group:UsingDefaultScale() then
        uab.main.secureheader:SetStateScaleRange(self.states.min, self.states.max)
    else
        uab.main.secureheader:SetStateScaleRange(self.states.min, self.states.max, self.group:GetScale())
    end
end

function uab.classes.UnitGroupHeader.prototype:PositionChanged()
    for _, control in pairs(self.controls) do
        control:PositionChanged()
    end
    
    if self.group:UsingDefaultPosition() then
        uab.main.secureheader:SetStatePositionRange(self.states.min, self.states.max)
    else
        uab.main.secureheader:SetStatePositionRange(self.states.min, self.states.max, self.group:GetPosition().x, self.group:GetPosition().y, "cursor", "CENTER")
    end
end

function uab.classes.UnitGroupHeader.prototype:StayOpenChanged()
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:StayOpenChanged()
	end	
end

function uab.classes.UnitGroupHeader.prototype:FastChangeUnitChanged()
    for _, control in pairs(self.controls) do
        control:StatesInUseChanged()
    end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.UnitGroupHeader.prototype:SetScale(frame)
    if frame then
        self.scaleframe = frame
    end

    local relativeScale = self:RelativeScale()

    self.frame:SetScale(relativeScale)
end

function uab.classes.UnitGroupHeader.prototype:RelativeScale()
    local relativeScale = self.group:GetScale()

    if self.scaleframe then
        local frame = self.scaleframe

        while frame:GetParent() do
            local nextref = frame:GetParent()
            if nextref then
                if nextref ~= UIParent then
                    frame = nextref
                else
                    do break end
                end
            end
        end
    
        local scale = 1.0 / frame:GetScale()
        
        relativeScale = scale * relativeScale
    end
    
    return relativeScale
end

function uab.classes.UnitGroupHeader.prototype:HaveAvailableControl()
    for _, control in pairs(self.controls) do
        if not control.unitframe then
            return true
        end
    end
end

function uab.classes.UnitGroupHeader.prototype:NextControl()
    for _, control in pairs(self.controls) do
        if not control.unitframe then
            return control
        end
    end
end
